function [w, o, delta] = pointmaptrain( Iv, Ov, gamma, Cmax)
% PointMap training (c) Norbert Kopco 2002
%
% Parameters:
% Iv: matrix of training inputs (one input per row)
% Ov: column vector of training outputs (numbers should be integers larger than 0)
% gamma: criticality parameter (0<=gamma<=1)
% Cmax: maximum code size parameter (Cmax>0)
%
% Returns:
% variables w, o, delta contain the learned code and information value of individual nodes

if size(Iv,1)<1            disp('Error: Input vector I must have at least one row!!!!');    w=[]; return; end
if size(Iv,1)~=size(Ov,1)  disp('Error: Input and output vector must have the sume number of rows!!!!');    w=[]; return; end
if min(Ov)<1               disp('o is assumed to include only integers larger than 0!!!!');    w=[]; return; end
if gamma<0 | gamma>1       disp('Error: gamma must be from range [0,1]!!!!');    w=[]; return; end
if Cmax<1                  disp('Error: Cmax must be larger than zero!!!!');    w=[]; return; end

% step 1:
C=1;
w=Iv(1,:);
o=Ov(1);
alpha(1)=0; beta(1)=0; xi(1)=0; delta(1)=0;
N=size(Iv,1);
M=size(Iv,2);

% step 2:
for m=2:N
    I=Iv(m,:);    O=Ov(m);
    
    % step 3:
    dist=sum(abs(repmat(I,C,1)-w),2);
    [aux,J]=min(dist); dist(J)=inf;
    
    % step 4:
    alpha(J)=alpha(J)+1;
    if O==o(J)
        beta(J)=beta(J)+1;
        if C<2
            xi(J)=xi(J)+1;
        else
            [aux,J2]=min(dist);
            if O~=o(J2)     xi(J)=xi(J)+1;    end;
        end
    end
    delta(J) = gamma * xi(J)/(beta(J)+1)+(1-gamma)*(beta(J)+0.5)/(alpha(J)+1);
    
    % step 5
    if O~=o(J)
        
        % step 6
        if C==Cmax
            [aux, K]= min(delta);
            w = w([1:K-1 K+1:end],:);             o = o([1:K-1 K+1:end]); 
            alpha = alpha([1:K-1 K+1:end]);       beta = beta([1:K-1 K+1:end]); 
            xi = xi([1:K-1 K+1:end]);             delta = delta([1:K-1 K+1:end]); 
            C=C-1;
        end
        
        % step 7
        C=C+1;
        w(C,:)=I;   o(C,1)=O;
        alpha(C,1)=0; beta(C,1)=0; xi(C,1)=0; delta(C,1)=0;
    end

%     uncomment this section if you wish to store the state of the system after presentation of a fixed number of inputs
%     here, state is stored after every 80000 training inputs
%     if 1.2500e-005*m==round(1.2500e-005*m) 
%         disp(sprintf('m=%d',m));
%         save(['state_gamma' num2str(gamma) '_Cmax' num2str(Cmax) '_' num2str(m) '.mat'], 'w', 'o', 'delta');
%     end

end
